import * as i0 from '@angular/core';
import { InjectionToken, Injectable, Inject, Injector, Optional, ChangeDetectorRef, Directive, ElementRef, Renderer2, Input, PLATFORM_ID, Pipe, NgModule, forwardRef } from '@angular/core';
import { __awaiter, __rest } from 'tslib';
import { of, throwError, BehaviorSubject, concat, merge, Subject } from 'rxjs';
import { shareReplay, takeUntil, filter } from 'rxjs/operators';
import { Location, isPlatformBrowser } from '@angular/common';
import { Router, NavigationStart, NavigationEnd } from '@angular/router';
import { NG_VALIDATORS } from '@angular/forms';

/**
 * L10n configuration token.
 */
const L10N_CONFIG = new InjectionToken('L10N_CONFIG');
/**
 * L10n locale token.
 */
const L10N_LOCALE = new InjectionToken('L10N_LOCALE');

function l10nError(type, value) {
    return `angular-l10n (${type.name}): ${value}`;
}

function validateLanguage(language) {
    const regExp = new RegExp(/^([a-z]{2,3})(\-[A-Z][a-z]{3})?(\-[A-Z]{2})?(-u.+)?$/);
    return regExp.test(language);
}
function formatLanguage(language, format) {
    if (language == null || language === '')
        return '';
    if (!validateLanguage(language))
        throw l10nError(formatLanguage, 'Invalid language');
    const [, LANGUAGE = '', SCRIPT = '', REGION = ''] = language.match(/^([a-z]{2,3})(\-[A-Z][a-z]{3})?(\-[A-Z]{2})?/) || [];
    switch (format) {
        case 'language':
            return LANGUAGE;
        case 'language-script':
            return LANGUAGE + SCRIPT;
        case 'language-region':
            return LANGUAGE + REGION;
        case 'language-script-region':
            return LANGUAGE + SCRIPT + REGION;
    }
}
function parseLanguage(language) {
    const groups = language.match(/^([a-z]{2,3})(\-([A-Z][a-z]{3}))?(\-([A-Z]{2}))?(-u.+)?$/);
    if (groups == null)
        throw l10nError(parseLanguage, 'Invalid language');
    return {
        language: groups[1],
        script: groups[3],
        region: groups[5],
        extension: groups[6]
    };
}
function getBrowserLanguage(format) {
    let browserLanguage = null;
    if (typeof navigator !== 'undefined' && navigator.language) {
        switch (format) {
            case 'language-region':
            case 'language-script-region':
                browserLanguage = navigator.language;
                break;
            default:
                browserLanguage = navigator.language.split('-')[0];
        }
    }
    return browserLanguage;
}
function getSchema(schema, language, format) {
    const element = schema.find(item => formatLanguage(item.locale.language, format) === language);
    return element;
}
function getValue(key, data, keySeparator) {
    if (data) {
        if (keySeparator) {
            return key.split(keySeparator).reduce((acc, cur) => (acc && acc[cur]) != null ? acc[cur] : null, data);
        }
        return data[key] != null ? data[key] : null;
    }
    return null;
}
function handleParams(value, params) {
    const replacedValue = value.replace(/{{\s?([^{}\s]*)\s?}}/g, (substring, parsedKey) => {
        const replacer = params[parsedKey];
        return replacer !== undefined ? replacer : substring;
    });
    return replacedValue;
}
function mergeDeep(target, source) {
    const output = Object.assign({}, target);
    if (isObject(target) && isObject(source)) {
        Object.keys(source).forEach((key) => {
            if (isObject(source[key])) {
                if (!(key in target)) {
                    Object.assign(output, { [key]: source[key] });
                }
                else {
                    output[key] = mergeDeep(target[key], source[key]);
                }
            }
            else {
                Object.assign(output, { [key]: source[key] });
            }
        });
    }
    return output;
}
function hasIntl() {
    const isAvailable = typeof Intl === 'object' && !!Intl;
    return isAvailable;
}
function hasDateTimeFormat() {
    return hasIntl() && Intl.hasOwnProperty('DateTimeFormat');
}
function hasNumberFormat() {
    return hasIntl() && Intl.hasOwnProperty('NumberFormat');
}
function hasTimeZone() {
    if (hasIntl() && hasDateTimeFormat()) {
        try {
            new Intl.DateTimeFormat('en-US', { timeZone: 'America/Los_Angeles' }).format(new Date());
        }
        catch (e) {
            return false;
        }
        return true;
    }
    return false;
}
function hasRelativeTimeFormat() {
    return hasIntl() && Intl.hasOwnProperty('RelativeTimeFormat');
}
function hasCollator() {
    return hasIntl() && Intl.hasOwnProperty('Collator');
}
function hasPluralRules() {
    return hasIntl() && Intl.hasOwnProperty('PluralRules');
}
function hasListFormat() {
    return hasIntl() && Intl.hasOwnProperty('ListFormat');
}
function hasDisplayNames() {
    return hasIntl() && Intl.hasOwnProperty('DisplayNames');
}
function toNumber(value) {
    const parsedValue = typeof value === 'string' && !isNaN(+value - parseFloat(value)) ? +value : value;
    return parsedValue;
}
function toDate(value) {
    if (isDate(value)) {
        return value;
    }
    if (typeof value === 'number' && !isNaN(value)) {
        return new Date(value);
    }
    if (typeof value === 'string') {
        value = value.trim();
        if (!isNaN(value - parseFloat(value))) {
            return new Date(parseFloat(value));
        }
        if (/^(\d{4}-\d{1,2}-\d{1,2})$/.test(value)) {
            const [y, m, d] = value.split('-').map((val) => +val);
            return new Date(y, m - 1, d);
        }
        const match = value.match(/^(\d{4})-?(\d\d)-?(\d\d)(?:T(\d\d)(?::?(\d\d)(?::?(\d\d)(?:\.(\d+))?)?)?(Z|([+-])(\d\d):?(\d\d))?)?$/);
        if (match) {
            return isoStringToDate(match);
        }
    }
    const date = new Date(value);
    if (!isDate(date)) {
        throw l10nError(toDate, 'Invalid date');
    }
    return date;
}
const PARSE_DATE_STYLE = {
    full: { weekday: 'long', year: 'numeric', month: 'long', day: 'numeric' },
    long: { year: 'numeric', month: 'long', day: 'numeric' },
    medium: { year: 'numeric', month: 'short', day: 'numeric' },
    short: { year: '2-digit', month: 'numeric', day: 'numeric' }
};
const PARSE_TIME_STYLE = {
    full: { hour: 'numeric', minute: 'numeric', second: 'numeric', timeZoneName: 'long' },
    long: { hour: 'numeric', minute: 'numeric', second: 'numeric', timeZoneName: 'short' },
    medium: { hour: 'numeric', minute: 'numeric', second: 'numeric' },
    short: { hour: 'numeric', minute: 'numeric' }
};
function parseDigits(digits) {
    const groups = digits.match(/^(\d+)?\.((\d+)(\-(\d+))?)?$/);
    if (groups == null)
        throw l10nError(parseDigits, 'Invalid digits');
    return {
        minimumIntegerDigits: groups[1] ? parseInt(groups[1]) : undefined,
        minimumFractionDigits: groups[3] ? parseInt(groups[3]) : undefined,
        maximumFractionDigits: groups[5] ? parseInt(groups[5]) : undefined,
    };
}
function isObject(item) {
    return typeof item === 'object' && !Array.isArray(item);
}
function isDate(value) {
    return value instanceof Date && !isNaN(value.valueOf());
}
/**
 * Converts a date in ISO 8601 to a Date.
 */
function isoStringToDate(match) {
    const date = new Date(0);
    let tzHour = 0;
    let tzMin = 0;
    const dateSetter = match[8] ? date.setUTCFullYear : date.setFullYear;
    const timeSetter = match[8] ? date.setUTCHours : date.setHours;
    if (match[9]) {
        tzHour = Number(match[9] + match[10]);
        tzMin = Number(match[9] + match[11]);
    }
    dateSetter.call(date, Number(match[1]), Number(match[2]) - 1, Number(match[3]));
    const h = Number(match[4] || 0) - tzHour;
    const m = Number(match[5] || 0) - tzMin;
    const s = Number(match[6] || 0);
    const ms = Math.round(parseFloat('0.' + (match[7] || 0)) * 1000);
    timeSetter.call(date, h, m, s, ms);
    return date;
}

class L10nCache {
    constructor() {
        this.cache = {};
    }
    read(key, request) {
        if (this.cache[key])
            return this.cache[key];
        const response = request.pipe(shareReplay(1));
        this.cache[key] = response;
        return response;
    }
}
L10nCache.decorators = [
    { type: Injectable }
];

/**
 * Implement this class-interface to create a storage for the locale.
 */
class L10nStorage {
}
L10nStorage.decorators = [
    { type: Injectable }
];
class L10nDefaultStorage {
    read() {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.resolve(null);
        });
    }
    write(locale) {
        return __awaiter(this, void 0, void 0, function* () { });
    }
}
L10nDefaultStorage.decorators = [
    { type: Injectable }
];

/**
 * Implement this class-interface to get the user language.
 */
class L10nUserLanguage {
}
L10nUserLanguage.decorators = [
    { type: Injectable }
];
class L10nDefaultUserLanguage {
    constructor(config) {
        this.config = config;
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            const browserLanguage = getBrowserLanguage(this.config.format);
            return Promise.resolve(browserLanguage);
        });
    }
}
L10nDefaultUserLanguage.decorators = [
    { type: Injectable }
];
L10nDefaultUserLanguage.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [L10N_CONFIG,] }] }
];

/**
 * Implement this class-interface to create a loader of translation data.
 */
class L10nTranslationLoader {
}
L10nTranslationLoader.decorators = [
    { type: Injectable }
];
class L10nDefaultTranslationLoader {
    get(language, provider) {
        return provider.asset[language] ?
            of(provider.asset[language]) :
            throwError(l10nError(L10nDefaultTranslationLoader, 'Asset not found'));
    }
}
L10nDefaultTranslationLoader.decorators = [
    { type: Injectable }
];

/**
 * Implement this class-interface to create a translation fallback.
 */
class L10nTranslationFallback {
}
L10nTranslationFallback.decorators = [
    { type: Injectable }
];
class L10nDefaultTranslationFallback {
    constructor(config, cache, translationLoader) {
        this.config = config;
        this.cache = cache;
        this.translationLoader = translationLoader;
    }
    /**
     * Translation data will be merged in the following order:
     * 'language'
     * 'language[-script]'
     * 'language[-script][-region]'
     */
    get(language, provider) {
        const loaders = [];
        const keywords = language.match(/-?[a-zA-z]+/g) || [];
        let fallbackLanguage = '';
        for (const keyword of keywords) {
            fallbackLanguage += keyword;
            if (this.config.cache) {
                loaders.push(this.cache.read(`${provider.name}-${fallbackLanguage}`, this.translationLoader.get(fallbackLanguage, provider)));
            }
            else {
                loaders.push(this.translationLoader.get(fallbackLanguage, provider));
            }
        }
        return loaders;
    }
}
L10nDefaultTranslationFallback.decorators = [
    { type: Injectable }
];
L10nDefaultTranslationFallback.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [L10N_CONFIG,] }] },
    { type: L10nCache },
    { type: L10nTranslationLoader }
];

/**
 * Implement this class-interface to create an handler for translated values.
 */
class L10nTranslationHandler {
}
L10nTranslationHandler.decorators = [
    { type: Injectable }
];
class L10nDefaultTranslationHandler {
    parseValue(key, params, value) {
        if (params)
            return handleParams(value, params);
        return value;
    }
}
L10nDefaultTranslationHandler.decorators = [
    { type: Injectable }
];

/**
 * Implement this class-interface to create an handler for missing values.
 */
class L10nMissingTranslationHandler {
}
L10nMissingTranslationHandler.decorators = [
    { type: Injectable }
];
class L10nDefaultMissingTranslationHandler {
    handle(key, value, params) {
        return key;
    }
}
L10nDefaultMissingTranslationHandler.decorators = [
    { type: Injectable }
];

/**
 * Implement this class-interface to change the localized path.
 */
class L10nLocation {
}
L10nLocation.decorators = [
    { type: Injectable }
];
class L10nDefaultLocation {
    constructor(config, injector) {
        this.config = config;
        this.injector = injector;
    }
    get location() {
        return this.injector.get(Location);
    }
    path() {
        return this.location.path(true);
    }
    parsePath(path) {
        if (!path)
            return null;
        const segment = this.getLocalizedSegment(path);
        if (segment != null) {
            const language = segment.replace(/\//g, '');
            return language;
        }
        return null;
    }
    getLocalizedSegment(path) {
        for (const element of this.config.schema) {
            const language = formatLanguage(element.locale.language, this.config.format);
            const regex = new RegExp(`(\/${language}\/)|(\/${language}$)|(\/(${language})(?=\\?))|(\/(${language})(?=\\#))`);
            const segments = path.match(regex);
            if (segments != null) {
                return segments[0];
            }
        }
        return null;
    }
    toLocalizedPath(language, path) {
        const segment = this.getLocalizedSegment(path);
        if (segment != null && segment.includes(language))
            return path;
        return Location.stripTrailingSlash('/' + language + path);
    }
}
L10nDefaultLocation.decorators = [
    { type: Injectable }
];
L10nDefaultLocation.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [L10N_CONFIG,] }] },
    { type: Injector }
];

class L10nTranslationService {
    constructor(config, locale, cache, storage, userLanguage, translationFallback, translationLoader, translationHandler, missingTranslationHandler, location) {
        this.config = config;
        this.locale = locale;
        this.cache = cache;
        this.storage = storage;
        this.userLanguage = userLanguage;
        this.translationFallback = translationFallback;
        this.translationLoader = translationLoader;
        this.translationHandler = translationHandler;
        this.missingTranslationHandler = missingTranslationHandler;
        this.location = location;
        /**
         * The translation data: {language: {key: value}}
         */
        this.data = {};
        this.translation = new BehaviorSubject(this.locale);
        this.error = new BehaviorSubject(null);
    }
    /**
     * Gets the current locale.
     */
    getLocale() {
        return this.locale;
    }
    /**
     * Changes the current locale and load the translation data.
     * @param locale The new locale
     */
    setLocale(locale) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.loadTranslation(this.config.providers, locale);
        });
    }
    /**
     * Fired every time the translation data has been loaded. Returns the locale.
     */
    onChange() {
        return this.translation.asObservable();
    }
    /**
     * Fired when the translation data could not been loaded. Returns the error.
     */
    onError() {
        return this.error.asObservable();
    }
    /**
     * Translates a key or an array of keys.
     * @param keys The key or an array of keys to be translated
     * @param params Optional parameters contained in the key
     * @param language The current language
     * @return The translated value or an object: {key: value}
     */
    translate(keys, params, language = this.locale.language) {
        language = formatLanguage(language, this.config.format);
        if (Array.isArray(keys)) {
            const data = {};
            for (const key of keys) {
                data[key] = this.translate(key, params, language);
            }
            return data;
        }
        const value = getValue(keys, this.data[language], this.config.keySeparator);
        return value ? this.translationHandler.parseValue(keys, params, value) : this.missingTranslationHandler.handle(keys, value, params);
    }
    /**
     * Checks if a translation exists.
     * @param key The key to be tested
     * @param language The current language
     */
    has(key, language = this.locale.language) {
        language = formatLanguage(language, this.config.format);
        return getValue(key, this.data[language], this.config.keySeparator) !== null;
    }
    /**
     * Gets the language direction.
     */
    getLanguageDirection(language = this.locale.language) {
        const schema = getSchema(this.config.schema, language, this.config.format);
        return schema ? schema.dir : undefined;
    }
    /**
     * Gets available languages.
     */
    getAvailableLanguages() {
        const languages = this.config.schema.map(item => formatLanguage(item.locale.language, this.config.format));
        return languages;
    }
    /**
     * Should only be called when the service instance is created.
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            let locale = null;
            // Tries to get locale from path if localized routing is used.
            if (this.location) {
                const path = this.location.path();
                const pathLanguage = yield this.location.parsePath(path);
                if (pathLanguage) {
                    const schema = getSchema(this.config.schema, pathLanguage, this.config.format);
                    if (schema) {
                        locale = schema.locale;
                    }
                }
            }
            // Tries to get locale from storage.
            if (locale == null) {
                locale = yield this.storage.read();
            }
            // Tries to get locale through the user language.
            if (locale == null) {
                const browserLanguage = yield this.userLanguage.get();
                if (browserLanguage) {
                    const schema = getSchema(this.config.schema, browserLanguage, this.config.format);
                    if (schema) {
                        locale = schema.locale;
                    }
                }
            }
            // Gets the default locale.
            if (locale == null) {
                locale = this.config.defaultLocale;
            }
            // Loads translation data.
            yield this.loadTranslation(this.config.providers, locale);
        });
    }
    /**
     * Can be called at every translation change.
     * @param providers An array of L10nProvider
     * @param locale The current locale
     */
    loadTranslation(providers = this.config.providers, locale = this.locale) {
        return __awaiter(this, void 0, void 0, function* () {
            // Updates providers
            providers.forEach(provider => {
                if (!this.config.providers.find(p => p.name === provider.name)) {
                    this.config.providers.push(provider);
                }
            });
            const language = formatLanguage(locale.language, this.config.format);
            return new Promise((resolve) => {
                concat(...this.getTranslation(providers, language)).subscribe({
                    next: (data) => this.addData(data, language),
                    error: (error) => {
                        this.handleError(error);
                        resolve();
                    },
                    complete: () => {
                        this.releaseTranslation(locale);
                        resolve();
                    }
                });
            });
        });
    }
    /**
     * Can be called to add translation data.
     * @param data The translation data {key: value}
     * @param language The language to add data
     */
    addData(data, language) {
        this.data[language] = this.data[language] !== undefined
            ? mergeDeep(this.data[language], data)
            : data;
    }
    getTranslation(providers, language) {
        const lazyLoaders = [];
        let loaders = [];
        for (const provider of providers) {
            if (this.config.fallback) {
                loaders = loaders.concat(this.translationFallback.get(language, provider));
            }
            else {
                if (this.config.cache) {
                    lazyLoaders.push(this.cache.read(`${provider.name}-${language}`, this.translationLoader.get(language, provider)));
                }
                else {
                    lazyLoaders.push(this.translationLoader.get(language, provider));
                }
            }
        }
        loaders.push(merge(...lazyLoaders));
        return loaders;
    }
    handleError(error) {
        this.error.next(error);
    }
    releaseTranslation(locale) {
        Object.assign(this.locale, locale);
        this.translation.next(this.locale);
        this.storage.write(this.locale);
    }
}
L10nTranslationService.decorators = [
    { type: Injectable }
];
L10nTranslationService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [L10N_CONFIG,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [L10N_LOCALE,] }] },
    { type: L10nCache },
    { type: L10nStorage },
    { type: L10nUserLanguage },
    { type: L10nTranslationFallback },
    { type: L10nTranslationLoader },
    { type: L10nTranslationHandler },
    { type: L10nMissingTranslationHandler },
    { type: L10nLocation, decorators: [{ type: Optional }] }
];

class L10nAsyncPipe {
    constructor(translation, cdr) {
        this.translation = translation;
        this.cdr = cdr;
        this.onChanges = this.translation.onChange().subscribe({
            next: () => this.cdr.markForCheck()
        });
    }
    ngOnDestroy() {
        if (this.onChanges)
            this.onChanges.unsubscribe();
    }
}
L10nAsyncPipe.decorators = [
    { type: Injectable }
];
L10nAsyncPipe.ctorParameters = () => [
    { type: L10nTranslationService },
    { type: ChangeDetectorRef }
];

/**
 * Breadth First Search (BFS) algorithm for traversing & searching tree data structure of DOM
 * explores the neighbor nodes first, before moving to the next level neighbors.
 * Time complexity: between O(1) and O(|V|^2).
 */
function getTargetNode(rootNode) {
    return walk(rootNode);
}
const MAX_DEPTH = 10;
function walk(rootNode) {
    const queue = [];
    let iNode;
    let depth = 0;
    let nodeToDepthIncrease = 1;
    queue.push(rootNode);
    while (queue.length > 0 && depth <= MAX_DEPTH) {
        iNode = queue.splice(0, 1)[0];
        if (isTargetNode(iNode))
            return iNode;
        if (depth < MAX_DEPTH && iNode.childNodes) {
            for (const child of Array.from(iNode.childNodes)) {
                if (isValidNode(child)) {
                    queue.push(child);
                }
            }
        }
        if (--nodeToDepthIncrease === 0) {
            depth++;
            nodeToDepthIncrease = queue.length;
        }
    }
    return rootNode;
}
function isTargetNode(node) {
    return typeof node !== 'undefined' && node.nodeType === 3 && node.nodeValue != null && node.nodeValue.trim() !== '';
}
/**
 * A valid node is not marked for translation.
 */
function isValidNode(node) {
    if (typeof node !== 'undefined' && node.nodeType === 1 && node.attributes) {
        for (const attr of Array.from(node.attributes)) {
            if (attr && /^l10n|translate/.test(attr.name))
                return false;
        }
    }
    return true;
}

class L10nDirective {
    constructor(el, renderer, translation) {
        this.el = el;
        this.renderer = renderer;
        this.translation = translation;
        this.value = null;
        this.language = undefined;
        this.content = null;
        this.text = null;
        this.attributes = [];
        this.element = null;
        this.renderNode = null;
        this.nodeValue = null;
        this.textObserver = null;
        this.destroy = new Subject();
    }
    set innerHTML(content) {
        // Handle TrustedHTML
        this.content = content.toString();
    }
    ngAfterViewInit() {
        if (this.el && this.el.nativeElement) {
            this.element = this.el.nativeElement;
            this.renderNode = getTargetNode(this.el.nativeElement);
            this.text = this.getText();
            this.attributes = this.getAttributes();
            this.addTextListener();
            if (this.language) {
                this.replaceText();
                this.replaceAttributes();
            }
            else {
                this.addTranslationListener();
            }
        }
    }
    ngOnChanges() {
        if (this.text) {
            if (this.nodeValue == null || this.nodeValue === '') {
                if (this.value) {
                    this.text = this.value;
                }
                else if (this.content) {
                    this.text = this.content;
                }
            }
            this.replaceText();
        }
        if (this.attributes && this.attributes.length > 0) {
            this.replaceAttributes();
        }
    }
    ngOnDestroy() {
        this.destroy.next(true);
        this.removeTextListener();
    }
    getText() {
        let text = '';
        if (this.element && this.element.childNodes.length > 0) {
            text = this.getNodeValue();
        }
        else if (this.value) {
            text = this.value;
        }
        else if (this.content) {
            text = this.content;
        }
        return text;
    }
    getNodeValue() {
        this.nodeValue = this.renderNode != null && this.renderNode.nodeValue != null ? this.renderNode.nodeValue : '';
        return this.nodeValue ? this.nodeValue.trim() : '';
    }
    getAttributes() {
        const attributes = [];
        if (this.element && this.element.attributes) {
            for (const attr of Array.from(this.element.attributes)) {
                if (attr && attr.name) {
                    const [, name = ''] = attr.name.match(/^l10n-(.+)$/) || [];
                    if (name) {
                        const targetAttr = Array.from(this.element.attributes).find(a => a.name === name);
                        if (targetAttr)
                            attributes.push({ name: targetAttr.name, value: targetAttr.value });
                    }
                }
            }
        }
        return attributes;
    }
    addTextListener() {
        if (typeof MutationObserver !== 'undefined') {
            this.textObserver = new MutationObserver(() => {
                if (this.element) {
                    this.renderNode = getTargetNode(this.element);
                    this.text = this.getText();
                    this.replaceText();
                }
            });
            if (this.renderNode) {
                this.textObserver.observe(this.renderNode, { subtree: true, characterData: true });
            }
        }
    }
    removeTextListener() {
        if (this.textObserver && typeof this.textObserver !== 'undefined') {
            this.textObserver.disconnect();
        }
    }
    addTranslationListener() {
        this.translation.onChange().pipe(takeUntil(this.destroy)).subscribe({
            next: () => {
                this.replaceText();
                this.replaceAttributes();
            }
        });
    }
    replaceText() {
        if (this.text) {
            this.setText(this.getValue(this.text));
        }
    }
    replaceAttributes() {
        if (this.attributes.length > 0) {
            this.setAttributes(this.getAttributesValues());
        }
    }
    setText(value) {
        if (value) {
            if (this.nodeValue && this.text) {
                this.removeTextListener();
                this.renderer.setValue(this.renderNode, this.nodeValue.replace(this.text, value));
                this.addTextListener();
            }
            else if (this.value) {
                this.renderer.setAttribute(this.element, 'value', value);
            }
            else if (this.content) {
                this.renderer.setProperty(this.element, 'innerHTML', value);
            }
        }
    }
    setAttributes(data) {
        for (const attr of this.attributes) {
            this.renderer.setAttribute(this.element, attr.name, data[attr.value]);
        }
    }
    getAttributesValues() {
        const values = this.attributes.map(attr => attr.value);
        const data = {};
        for (const value of values) {
            data[value] = this.getValue(value);
        }
        return data;
    }
}
L10nDirective.decorators = [
    { type: Directive }
];
L10nDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: L10nTranslationService }
];
L10nDirective.propDecorators = {
    value: [{ type: Input }],
    innerHTML: [{ type: Input }],
    language: [{ type: Input }]
};

class L10nResolver {
    constructor(translation) {
        this.translation = translation;
    }
    resolve(route, state) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.translation.init();
            yield this.translation.loadTranslation(route.data.l10nProviders);
        });
    }
}
L10nResolver.ɵprov = i0.ɵɵdefineInjectable({ factory: function L10nResolver_Factory() { return new L10nResolver(i0.ɵɵinject(L10nTranslationService)); }, token: L10nResolver, providedIn: "root" });
L10nResolver.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
L10nResolver.ctorParameters = () => [
    { type: L10nTranslationService }
];

class L10nRoutingService {
    constructor(platformId, config, locale, translation, l10nLocation, injector) {
        this.platformId = platformId;
        this.config = config;
        this.locale = locale;
        this.translation = translation;
        this.l10nLocation = l10nLocation;
        this.injector = injector;
    }
    get router() {
        return this.injector.get(Router);
    }
    get location() {
        return this.injector.get(Location);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            // Parses the url to find the language when a navigation starts.
            this.router.events.pipe(filter((event) => event instanceof NavigationStart)).subscribe({
                next: (event) => {
                    // Skips location change on pop state event and on first navigation.
                    this.redirectToPath(event.url, event.navigationTrigger === 'popstate' || event.id === 1);
                }
            });
            // Replaces url when a navigation ends.
            this.router.events.pipe(filter((event) => event instanceof NavigationEnd)).subscribe({
                next: (event) => {
                    const url = (event.url && event.url !== '/' && event.url === event.urlAfterRedirects) ?
                        event.url :
                        event.urlAfterRedirects;
                    this.replacePath(this.locale, url);
                }
            });
            if (isPlatformBrowser(this.platformId)) {
                // Replaces url when locale changes.
                this.translation.onChange().subscribe({
                    next: (locale) => this.replacePath(locale)
                });
            }
        });
    }
    /**
     * Removes the language from the path and navigates.
     * @param path Localized path
     * @param skipLocationChange When true, navigates without pushing a new state into history
     */
    redirectToPath(path, skipLocationChange) {
        const segment = this.l10nLocation.getLocalizedSegment(path);
        if (segment != null) {
            const url = path.replace(segment, '/');
            // navigateByUrl keeps the query params.
            this.router.navigateByUrl(url, { skipLocationChange });
        }
    }
    /**
     * Replaces the path with the language without pushing a new state into history.
     * @param locale The current locale
     * @param path The path to be replaced
     */
    replacePath(locale, path) {
        if (locale.language === '')
            return;
        const language = formatLanguage(locale.language, this.config.format);
        if (path) {
            if (!this.isDefaultRouting()) {
                this.location.replaceState(this.l10nLocation.toLocalizedPath(language, path));
            }
        }
        else {
            path = this.l10nLocation.path();
            const segment = this.l10nLocation.getLocalizedSegment(path);
            if (segment != null) {
                path = path.replace(segment, '/');
                if (this.isDefaultRouting()) {
                    this.location.replaceState(path);
                }
            }
            if (!this.isDefaultRouting()) {
                this.location.replaceState(this.l10nLocation.toLocalizedPath(language, path));
            }
        }
    }
    isDefaultRouting() {
        if (!this.config.defaultRouting)
            return false;
        return formatLanguage(this.locale.language, this.config.format) ===
            formatLanguage(this.config.defaultLocale.language, this.config.format);
    }
}
L10nRoutingService.decorators = [
    { type: Injectable }
];
L10nRoutingService.ctorParameters = () => [
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [L10N_CONFIG,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [L10N_LOCALE,] }] },
    { type: L10nTranslationService },
    { type: L10nLocation },
    { type: Injector }
];

/**
 * Implement this class-interface to init L10n.
 */
class L10nLoader {
}
L10nLoader.decorators = [
    { type: Injectable }
];
class L10nDefaultLoader {
    constructor(translation) {
        this.translation = translation;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.translation.init();
        });
    }
}
L10nDefaultLoader.decorators = [
    { type: Injectable }
];
L10nDefaultLoader.ctorParameters = () => [
    { type: L10nTranslationService }
];
class L10nRoutingLoader {
    constructor(routing, translation) {
        this.routing = routing;
        this.translation = translation;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.routing.init();
            yield this.translation.init();
        });
    }
}
L10nRoutingLoader.decorators = [
    { type: Injectable }
];
L10nRoutingLoader.ctorParameters = () => [
    { type: L10nRoutingService },
    { type: L10nTranslationService }
];

class L10nTranslatePipe {
    constructor(translation) {
        this.translation = translation;
    }
    transform(key, language, params) {
        if (key == null || key === '')
            return null;
        return this.translation.translate(key, params, language);
    }
}
L10nTranslatePipe.decorators = [
    { type: Pipe, args: [{
                name: 'translate',
                pure: true
            },] }
];
L10nTranslatePipe.ctorParameters = () => [
    { type: L10nTranslationService }
];
class L10nTranslateAsyncPipe extends L10nAsyncPipe {
    constructor(translation, cdr) {
        super(translation, cdr);
        this.translation = translation;
        this.cdr = cdr;
    }
    transform(key, params, language) {
        if (key == null || key === '')
            return null;
        return this.translation.translate(key, params, language);
    }
}
L10nTranslateAsyncPipe.decorators = [
    { type: Pipe, args: [{
                name: 'translateAsync',
                pure: false
            },] }
];
L10nTranslateAsyncPipe.ctorParameters = () => [
    { type: L10nTranslationService },
    { type: ChangeDetectorRef }
];

class L10nTranslateDirective extends L10nDirective {
    constructor(el, renderer, translation) {
        super(el, renderer, translation);
        this.el = el;
        this.renderer = renderer;
        this.translation = translation;
    }
    set l10nTranslate(params) {
        this.params = params;
    }
    set translate(params) {
        this.params = params;
    }
    getValue(text) {
        return this.translation.translate(text, this.params, this.language);
    }
}
L10nTranslateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[l10nTranslate],[translate]'
            },] }
];
L10nTranslateDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: L10nTranslationService }
];
L10nTranslateDirective.propDecorators = {
    l10nTranslate: [{ type: Input }],
    translate: [{ type: Input }],
    params: [{ type: Input }]
};

class L10nTranslationModule {
    static forRoot(config, token = {}) {
        return {
            ngModule: L10nTranslationModule,
            providers: [
                L10nTranslationService,
                L10nCache,
                { provide: L10N_CONFIG, useValue: config },
                { provide: L10N_LOCALE, useValue: { language: '' } },
                { provide: L10nStorage, useClass: token.storage || L10nDefaultStorage },
                { provide: L10nUserLanguage, useClass: token.userLanguage || L10nDefaultUserLanguage },
                { provide: L10nTranslationFallback, useClass: token.translationFallback || L10nDefaultTranslationFallback },
                { provide: L10nTranslationLoader, useClass: token.translationLoader || L10nDefaultTranslationLoader },
                { provide: L10nTranslationHandler, useClass: token.translationHandler || L10nDefaultTranslationHandler },
                {
                    provide: L10nMissingTranslationHandler,
                    useClass: token.missingTranslationHandler || L10nDefaultMissingTranslationHandler
                },
                { provide: L10nLoader, useClass: L10nDefaultLoader }
            ]
        };
    }
}
L10nTranslationModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    L10nTranslatePipe,
                    L10nTranslateAsyncPipe,
                    L10nTranslateDirective
                ],
                exports: [
                    L10nTranslatePipe,
                    L10nTranslateAsyncPipe,
                    L10nTranslateDirective
                ]
            },] }
];

class L10nIntlService {
    constructor(config, locale, translation) {
        this.config = config;
        this.locale = locale;
        this.translation = translation;
    }
    /**
     * Formats a date.
     * @param value A date, a number (milliseconds since UTC epoch) or an ISO 8601 string
     * @param options A L10n or Intl DateTimeFormatOptions object
     * @param language The current language
     * @param timeZone The current time zone
     */
    formatDate(value, options, language = this.locale.dateLanguage || this.locale.language, timeZone = this.locale.timeZone) {
        if (!hasDateTimeFormat() || language == null || language === '')
            return value;
        value = toDate(value);
        let dateTimeFormatOptions = {};
        if (options) {
            if (options) {
                const { dateStyle, timeStyle } = options, rest = __rest(options, ["dateStyle", "timeStyle"]);
                if (dateStyle) {
                    dateTimeFormatOptions = Object.assign(Object.assign({}, dateTimeFormatOptions), PARSE_DATE_STYLE[dateStyle]);
                }
                if (timeStyle) {
                    dateTimeFormatOptions = Object.assign(Object.assign({}, dateTimeFormatOptions), PARSE_TIME_STYLE[timeStyle]);
                }
                dateTimeFormatOptions = Object.assign(Object.assign({}, dateTimeFormatOptions), rest);
            }
        }
        if (hasTimeZone() && timeZone) {
            dateTimeFormatOptions.timeZone = timeZone;
        }
        return new Intl.DateTimeFormat(language, dateTimeFormatOptions).format(value);
    }
    /**
     * Formats a number.
     * @param value A number or a string
     * @param options A L10n or Intl NumberFormatOptions object
     * @param language The current language
     * @param currency The current currency
     */
    formatNumber(value, options, language = this.locale.numberLanguage || this.locale.language, currency = this.locale.currency) {
        if (!hasNumberFormat() && options && options.style === 'currency')
            return `${value} ${currency}`;
        if (!hasNumberFormat() || language == null || language === '')
            return value;
        value = toNumber(value);
        let numberFormatOptions = {};
        if (options) {
            const { digits } = options, rest = __rest(options, ["digits"]);
            if (digits) {
                numberFormatOptions = Object.assign(Object.assign({}, numberFormatOptions), parseDigits(digits));
            }
            numberFormatOptions = Object.assign(Object.assign({}, numberFormatOptions), rest);
        }
        if (currency)
            numberFormatOptions.currency = currency;
        return new Intl.NumberFormat(language, numberFormatOptions).format(value);
    }
    /**
     * Formats a relative time.
     * @param value A negative (or positive) number
     * @param unit The unit of the value
     * @param options An Intl RelativeTimeFormatOptions object
     * @param language The current language
     */
    formatRelativeTime(value, unit, options, language = this.locale.dateLanguage || this.locale.language) {
        if (!hasRelativeTimeFormat() || language == null || language === '')
            return value;
        value = toNumber(value);
        return new Intl.RelativeTimeFormat(language, options).format(value, unit);
    }
    getCurrencySymbol(locale = this.locale) {
        if (!hasNumberFormat())
            return locale.currency;
        const decimal = this.formatNumber(0, { digits: '1.0-0' }, locale.numberLanguage || locale.language);
        const currency = this.formatNumber(0, { digits: '1.0-0', style: 'currency', currencyDisplay: 'symbol' }, locale.numberLanguage || locale.language, locale.currency);
        let symbol = currency.replace(decimal, '');
        symbol = symbol.trim();
        return symbol;
    }
    /**
     * Compares two keys by the value of translation.
     * @param key1 First key to compare
     * @param key1 Second key to compare
     * @param options An Intl CollatorOptions object
     * @param language The current language
     * @return A negative value if the value of translation of key1 comes before the value of translation of key2;
     *         a positive value if key1 comes after key2;
     *         0 if they are considered equal or Intl.Collator is not supported
     */
    compare(key1, key2, options, language = this.locale.language) {
        if (!hasCollator() || language == null || language === '')
            return 0;
        const value1 = this.translation.translate(key1);
        const value2 = this.translation.translate(key2);
        return new Intl.Collator(language, options).compare(value1, value2);
    }
    /**
     * Gets the plural by a number.
     * @param value The number to get the plural
     * @param options An Intl PluralRulesOptions object
     * @param prefix Optional prefix for the key
     * @param language The current language
     */
    plural(value, prefix = '', options, language = this.locale.language) {
        if (!hasPluralRules() || language == null || language === '')
            return value.toString();
        value = toNumber(value);
        const rule = new Intl.PluralRules(language, options).select(value);
        const key = prefix ? `${prefix}${this.config.keySeparator}${rule}` : rule;
        return this.translation.translate(key);
    }
    /**
     * Returns the representation of a list.
     * @param list An array of keys
     * @param options An Intl ListFormatOptions object
     * @param language The current language
     */
    list(list, options, language = this.locale.language) {
        const values = list.map(key => this.translation.translate(key));
        if (!hasListFormat() || language == null || language === '')
            return values.join(', ');
        return new Intl.ListFormat(language, options).format(values);
    }
    /**
     * Returns translation of language, region, script or currency display names
     * @param code ISO code of language, region, script or currency
     * @param options A DisplayNamesOptions object
     * @param language The current language
     */
    displayNames(code, options, language = this.locale.language) {
        if (!hasDisplayNames() || language == null || language === '')
            return code;
        return new Intl.DisplayNames(language, options).of(code);
    }
}
L10nIntlService.decorators = [
    { type: Injectable }
];
L10nIntlService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [L10N_CONFIG,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [L10N_LOCALE,] }] },
    { type: L10nTranslationService }
];

class L10nDatePipe {
    constructor(intl) {
        this.intl = intl;
    }
    transform(value, language, options, timezone) {
        if (value == null || value === '')
            return null;
        return this.intl.formatDate(value, options, language, timezone);
    }
}
L10nDatePipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nDate',
                pure: true
            },] }
];
L10nDatePipe.ctorParameters = () => [
    { type: L10nIntlService }
];
class L10nDateAsyncPipe extends L10nAsyncPipe {
    constructor(translation, cdr, intl) {
        super(translation, cdr);
        this.translation = translation;
        this.cdr = cdr;
        this.intl = intl;
    }
    transform(value, options, language, timezone) {
        if (value == null || value === '')
            return null;
        return this.intl.formatDate(value, options, language, timezone);
    }
}
L10nDateAsyncPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nDateAsync',
                pure: false
            },] }
];
L10nDateAsyncPipe.ctorParameters = () => [
    { type: L10nTranslationService },
    { type: ChangeDetectorRef },
    { type: L10nIntlService }
];

class L10nNumberPipe {
    constructor(intl) {
        this.intl = intl;
    }
    transform(value, language, options, currency) {
        if (value == null || value === '')
            return null;
        return this.intl.formatNumber(value, options, language, currency);
    }
}
L10nNumberPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nNumber',
                pure: true
            },] }
];
L10nNumberPipe.ctorParameters = () => [
    { type: L10nIntlService }
];
class L10nNumberAsyncPipe extends L10nAsyncPipe {
    constructor(translation, cdr, intl) {
        super(translation, cdr);
        this.translation = translation;
        this.cdr = cdr;
        this.intl = intl;
    }
    transform(value, options, language, currency) {
        if (value == null || value === '')
            return null;
        return this.intl.formatNumber(value, options, language, currency);
    }
}
L10nNumberAsyncPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nNumberAsync',
                pure: false
            },] }
];
L10nNumberAsyncPipe.ctorParameters = () => [
    { type: L10nTranslationService },
    { type: ChangeDetectorRef },
    { type: L10nIntlService }
];

class L10nTimeAgoPipe {
    constructor(intl) {
        this.intl = intl;
    }
    transform(value, language, unit, options) {
        if (value == null || value === '')
            return null;
        return this.intl.formatRelativeTime(value, unit, options, language);
    }
}
L10nTimeAgoPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nTimeAgo',
                pure: true
            },] }
];
L10nTimeAgoPipe.ctorParameters = () => [
    { type: L10nIntlService }
];
class L10nTimeAgoAsyncPipe extends L10nAsyncPipe {
    constructor(translation, cdr, intl) {
        super(translation, cdr);
        this.translation = translation;
        this.cdr = cdr;
        this.intl = intl;
    }
    transform(value, unit, options, language) {
        if (value == null || value === '')
            return null;
        return this.intl.formatRelativeTime(value, unit, options, language);
    }
}
L10nTimeAgoAsyncPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nTimeAgoAsync',
                pure: false
            },] }
];
L10nTimeAgoAsyncPipe.ctorParameters = () => [
    { type: L10nTranslationService },
    { type: ChangeDetectorRef },
    { type: L10nIntlService }
];

class L10nPluralPipe {
    constructor(intl) {
        this.intl = intl;
    }
    transform(value, language, prefix, options) {
        if (value == null || value === '')
            return null;
        return this.intl.plural(value, prefix, options, language);
    }
}
L10nPluralPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nPlural',
                pure: true
            },] }
];
L10nPluralPipe.ctorParameters = () => [
    { type: L10nIntlService }
];
class L10nPluralAsyncPipe extends L10nAsyncPipe {
    constructor(translation, cdr, intl) {
        super(translation, cdr);
        this.translation = translation;
        this.cdr = cdr;
        this.intl = intl;
    }
    transform(value, language, prefix, options) {
        if (value == null || value === '')
            return null;
        return this.intl.plural(value, prefix, options, language);
    }
}
L10nPluralAsyncPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nPluralAsync',
                pure: false
            },] }
];
L10nPluralAsyncPipe.ctorParameters = () => [
    { type: L10nTranslationService },
    { type: ChangeDetectorRef },
    { type: L10nIntlService }
];

class L10nDisplayNamesPipe {
    constructor(intl) {
        this.intl = intl;
    }
    transform(value, language, options) {
        if (value == null || value === '')
            return null;
        return this.intl.displayNames(value, options, language);
    }
}
L10nDisplayNamesPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nDisplayNames',
                pure: true
            },] }
];
L10nDisplayNamesPipe.ctorParameters = () => [
    { type: L10nIntlService }
];
class L10nDisplayNamesAsyncPipe extends L10nAsyncPipe {
    constructor(translation, cdr, intl) {
        super(translation, cdr);
        this.translation = translation;
        this.cdr = cdr;
        this.intl = intl;
    }
    transform(value, options, language) {
        if (value == null || value === '')
            return null;
        return this.intl.displayNames(value, options, language);
    }
}
L10nDisplayNamesAsyncPipe.decorators = [
    { type: Pipe, args: [{
                name: 'l10nDisplayNamesAsync',
                pure: false
            },] }
];
L10nDisplayNamesAsyncPipe.ctorParameters = () => [
    { type: L10nTranslationService },
    { type: ChangeDetectorRef },
    { type: L10nIntlService }
];

class L10nDateDirective extends L10nDirective {
    constructor(el, renderer, translation, intl) {
        super(el, renderer, translation);
        this.el = el;
        this.renderer = renderer;
        this.translation = translation;
        this.intl = intl;
        this.timezone = undefined;
    }
    set l10nDate(options) {
        this.options = options;
    }
    getValue(text) {
        return this.intl.formatDate(text, this.options, this.language, this.timezone);
    }
}
L10nDateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[l10nDate]'
            },] }
];
L10nDateDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: L10nTranslationService },
    { type: L10nIntlService }
];
L10nDateDirective.propDecorators = {
    l10nDate: [{ type: Input }],
    options: [{ type: Input }],
    timezone: [{ type: Input }]
};

class L10nNumberDirective extends L10nDirective {
    constructor(el, renderer, translation, intl) {
        super(el, renderer, translation);
        this.el = el;
        this.renderer = renderer;
        this.translation = translation;
        this.intl = intl;
        this.currency = undefined;
    }
    set l10nNumber(options) {
        this.options = options;
    }
    getValue(text) {
        return this.intl.formatNumber(text, this.options, this.language, this.currency);
    }
}
L10nNumberDirective.decorators = [
    { type: Directive, args: [{
                selector: '[l10nNumber]'
            },] }
];
L10nNumberDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: L10nTranslationService },
    { type: L10nIntlService }
];
L10nNumberDirective.propDecorators = {
    l10nNumber: [{ type: Input }],
    options: [{ type: Input }],
    currency: [{ type: Input }]
};

class L10nTimeAgoDirective extends L10nDirective {
    constructor(el, renderer, translation, intl) {
        super(el, renderer, translation);
        this.el = el;
        this.renderer = renderer;
        this.translation = translation;
        this.intl = intl;
    }
    set l10nTimeAgo(options) {
        this.options = options;
    }
    getValue(text) {
        return this.intl.formatRelativeTime(text, this.unit, this.options, this.language);
    }
}
L10nTimeAgoDirective.decorators = [
    { type: Directive, args: [{
                selector: '[l10nTimeAgo]'
            },] }
];
L10nTimeAgoDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: L10nTranslationService },
    { type: L10nIntlService }
];
L10nTimeAgoDirective.propDecorators = {
    l10nTimeAgo: [{ type: Input }],
    unit: [{ type: Input }],
    options: [{ type: Input }]
};

class L10nIntlModule {
}
L10nIntlModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    L10nDatePipe,
                    L10nNumberPipe,
                    L10nTimeAgoPipe,
                    L10nPluralPipe,
                    L10nDisplayNamesPipe,
                    L10nDateAsyncPipe,
                    L10nNumberAsyncPipe,
                    L10nTimeAgoAsyncPipe,
                    L10nPluralAsyncPipe,
                    L10nDisplayNamesAsyncPipe,
                    L10nDateDirective,
                    L10nNumberDirective,
                    L10nTimeAgoDirective
                ],
                exports: [
                    L10nDatePipe,
                    L10nNumberPipe,
                    L10nTimeAgoPipe,
                    L10nPluralPipe,
                    L10nDisplayNamesPipe,
                    L10nDateAsyncPipe,
                    L10nNumberAsyncPipe,
                    L10nTimeAgoAsyncPipe,
                    L10nPluralAsyncPipe,
                    L10nDisplayNamesAsyncPipe,
                    L10nDateDirective,
                    L10nNumberDirective,
                    L10nTimeAgoDirective
                ],
                providers: [L10nIntlService]
            },] }
];

/**
 * Implement this class-interface to create a validation service.
 */
class L10nValidation {
}
L10nValidation.decorators = [
    { type: Injectable }
];
class L10nDefaultValidation {
    constructor(locale) {
        this.locale = locale;
    }
    parseNumber(value, options, language = this.locale.numberLanguage || this.locale.language) {
        return null;
    }
    parseDate(value, options, language = this.locale.dateLanguage || this.locale.language) {
        return null;
    }
}
L10nDefaultValidation.decorators = [
    { type: Injectable }
];
L10nDefaultValidation.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [L10N_LOCALE,] }] }
];

/**
 * Function that takes a control and returns either null when it’s valid, or an error object if it’s not.
 * @param validation The instance of L10nValidation service
 * @param options A L10n or Intl NumberFormatOptions object
 * @param minValue The minimum value
 * @param maxValue The maximum value
 * @param language The current language
 * @return An error object: 'format', 'minValue' or 'maxValue'; null in case the value is valid
 */
function l10nValidateNumber(validation, options, minValue = Number.MIN_VALUE, maxValue = Number.MAX_VALUE, language) {
    const validator = (c) => {
        if (c.value === '' || c.value == null)
            return null;
        const value = validation.parseNumber(c.value, options, language);
        if (value != null) {
            if (value < minValue) {
                return { minValue: true };
            }
            else if (value > maxValue) {
                return { maxValue: true };
            }
            return null; // The number is valid.
        }
        else {
            return { format: true };
        }
    };
    return validator;
}
class L10nValidateNumberDirective {
    constructor(validation) {
        this.validation = validation;
        this.minValue = undefined;
        this.maxValue = undefined;
        this.language = undefined;
        this.validator = null;
    }
    set l10nValidateNumber(options) {
        this.options = options;
    }
    ngOnInit() {
        this.validator = l10nValidateNumber(this.validation, this.options, this.minValue, this.maxValue, this.language);
    }
    ngOnChanges() {
        this.validator = l10nValidateNumber(this.validation, this.options, this.minValue, this.maxValue, this.language);
    }
    validate(c) {
        return this.validator ? this.validator(c) : null;
    }
}
L10nValidateNumberDirective.decorators = [
    { type: Directive, args: [{
                selector: '[l10nValidateNumber][ngModel],[l10nValidateNumber][formControl],[l10nValidateNumber][formControlName]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: forwardRef(() => L10nValidateNumberDirective), multi: true }
                ]
            },] }
];
L10nValidateNumberDirective.ctorParameters = () => [
    { type: L10nValidation }
];
L10nValidateNumberDirective.propDecorators = {
    l10nValidateNumber: [{ type: Input }],
    options: [{ type: Input }],
    minValue: [{ type: Input }],
    maxValue: [{ type: Input }],
    language: [{ type: Input }]
};

/**
 * Function that takes a control and returns either null when it’s valid, or an error object if it’s not.
 * @param validation The instance of L10nValidation service
 * @param options A L10n or Intl DateTimeFormatOptions object
 * @param minDate The minimum date
 * @param maxDate The maximum date
 * @param language The current language
 * @return An error object: 'format', 'minDate' or 'maxDate'; null in case the date is valid
 */
function l10nValidateDate(validation, options, minDate, maxDate, language) {
    const validator = (c) => {
        if (c.value === '' || c.value == null)
            return null;
        const date = validation.parseDate(c.value, options, language);
        if (date != null) {
            if (minDate && date < minDate) {
                return { mindate: true };
            }
            else if (maxDate && date > maxDate) {
                return { maxDate: true };
            }
            return null; // The date is valid.
        }
        else {
            return { format: true };
        }
    };
    return validator;
}
class L10nValidateDateDirective {
    constructor(validation) {
        this.validation = validation;
        this.minDate = undefined;
        this.maxDate = undefined;
        this.language = undefined;
        this.validator = null;
    }
    set l10nValidateDate(options) {
        this.options = options;
    }
    ngOnInit() {
        this.validator = l10nValidateDate(this.validation, this.options, this.minDate, this.maxDate, this.language);
    }
    ngOnChanges() {
        this.validator = l10nValidateDate(this.validation, this.options, this.minDate, this.maxDate, this.language);
    }
    validate(c) {
        return this.validator ? this.validator(c) : null;
    }
}
L10nValidateDateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[l10nValidateDate][ngModel],[l10nValidateDate][formControl],[l10nValidateDate][formControlName]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: forwardRef(() => L10nValidateDateDirective), multi: true }
                ]
            },] }
];
L10nValidateDateDirective.ctorParameters = () => [
    { type: L10nValidation }
];
L10nValidateDateDirective.propDecorators = {
    l10nValidateDate: [{ type: Input }],
    options: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    language: [{ type: Input }]
};

class L10nValidationModule {
    static forRoot(token = {}) {
        return {
            ngModule: L10nValidationModule,
            providers: [
                { provide: L10nValidation, useClass: token.validation || L10nDefaultValidation }
            ]
        };
    }
}
L10nValidationModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    L10nValidateNumberDirective,
                    L10nValidateDateDirective
                ],
                exports: [
                    L10nValidateNumberDirective,
                    L10nValidateDateDirective
                ]
            },] }
];

class L10nRoutingModule {
    static forRoot(token = {}) {
        return {
            ngModule: L10nRoutingModule,
            providers: [
                L10nRoutingService,
                { provide: L10nLocation, useClass: token.location || L10nDefaultLocation },
                { provide: L10nLoader, useClass: L10nRoutingLoader }
            ]
        };
    }
}
L10nRoutingModule.decorators = [
    { type: NgModule, args: [{},] }
];

/*
 * Public API Surface of angular-l10n
 */

/**
 * Generated bundle index. Do not edit.
 */

export { L10N_CONFIG, L10N_LOCALE, L10nAsyncPipe, L10nCache, L10nDateAsyncPipe, L10nDateDirective, L10nDatePipe, L10nDirective, L10nDisplayNamesAsyncPipe, L10nDisplayNamesPipe, L10nIntlModule, L10nIntlService, L10nLoader, L10nLocation, L10nMissingTranslationHandler, L10nNumberAsyncPipe, L10nNumberDirective, L10nNumberPipe, L10nPluralAsyncPipe, L10nPluralPipe, L10nResolver, L10nRoutingModule, L10nRoutingService, L10nStorage, L10nTimeAgoAsyncPipe, L10nTimeAgoDirective, L10nTimeAgoPipe, L10nTranslateAsyncPipe, L10nTranslateDirective, L10nTranslatePipe, L10nTranslationFallback, L10nTranslationHandler, L10nTranslationLoader, L10nTranslationModule, L10nTranslationService, L10nUserLanguage, L10nValidateDateDirective, L10nValidateNumberDirective, L10nValidation, L10nValidationModule, PARSE_DATE_STYLE, PARSE_TIME_STYLE, formatLanguage, getBrowserLanguage, getSchema, getValue, handleParams, hasCollator, hasDateTimeFormat, hasDisplayNames, hasIntl, hasListFormat, hasNumberFormat, hasPluralRules, hasRelativeTimeFormat, hasTimeZone, l10nValidateDate, l10nValidateNumber, mergeDeep, parseDigits, parseLanguage, toDate, toNumber, validateLanguage, L10nDefaultStorage as ɵa, L10nDefaultUserLanguage as ɵb, L10nDefaultTranslationFallback as ɵc, L10nDefaultTranslationLoader as ɵd, L10nDefaultTranslationHandler as ɵe, L10nDefaultMissingTranslationHandler as ɵf, L10nDefaultLoader as ɵg, L10nRoutingLoader as ɵh, L10nDefaultValidation as ɵi, L10nDefaultLocation as ɵj };
//# sourceMappingURL=angular-l10n.js.map
